@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixButton.c,v 1.11.2.3 2003/04/26 02:58:17 hobbs Exp $
+ * RCS: @(#) $Id: tkUnixButton.c,v 1.11.2.4 2003/10/10 20:20:47 hobbs Exp $
  */
 
 #include "tkButton.h"
@@ -293,15 +293,15 @@ TkpDisplayButton(clientData)
 		y += offset;
 	    }
 	    imageXOffset += x;
-	    imageXOffset += y;
+	    imageYOffset += y;
 	    if (butPtr->image != NULL) {
 		if ((butPtr->selectImage != NULL) &&
 			(butPtr->flags & SELECTED)) {
 		    Tk_RedrawImage(butPtr->selectImage, 0, 0, width,
-			    height, pixmap, x, y);
+			    height, pixmap, imageXOffset, imageYOffset);
 		} else {
 		    Tk_RedrawImage(butPtr->image, 0, 0, width, height, pixmap,
-			    x, y);
+			    imageXOffset, imageYOffset);
 		}
 	    } else {
 		XSetClipOrigin(butPtr->display, gc, x, y);
